/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_BIT_SELECT_STRATEGY__
#define __DD_BIT_SELECT_STRATEGY__

#include <vector>

#include "DDRayReference.h"
#include "DDBitCountVisitor.h"

namespace double_description {

    class BitSelectStrategy {

        protected:
        const unsigned long mEntries;
        const unsigned int mPlane;  

        const std::vector<unsigned long> mPopulationStats;

        std::vector<unsigned long> createPopulationStats ( const std::deque<EdgeCandidate> & entries ) const {
            std::vector<unsigned long> stats( mPlane, 0 );

            for ( unsigned long i = 0; i < entries.size(); i++ ) {
                for ( unsigned short bit = 0; bit < stats.size(); bit++ ) {
                    if ( entries[i].activeSet.isSet( bit ) ) {
                        stats[bit]++;
                    }
                }
            }
            
            return stats;
        }

        std::vector<unsigned long> createPopulationStats ( const RayReferenceContainer & entries ) const {

            BitCountVisitor visitor( mPlane );

            entries.iterate( visitor );
            
            return visitor.stats();
        }
        
        public:
        BitSelectStrategy ( const RayReferenceContainer & entries,
                            const unsigned int plane ) :
            mEntries( entries.size() ),
            mPlane( plane ),
            mPopulationStats( createPopulationStats( entries ) ) {}     

        BitSelectStrategy ( const std::deque<EdgeCandidate> & entries,
                            const unsigned int plane ) :
            mEntries( entries.size() ),
            mPlane( plane ),
            mPopulationStats( createPopulationStats( entries ) ) {} 

        virtual ~BitSelectStrategy () {}

        virtual std::vector<unsigned int> createMapping () const = 0;

        const std::vector<unsigned long> & getPopulationStats () const {
            return mPopulationStats;
        }

        const std::vector<double> calcBitOccurenceProbability () const {
            std::vector<double> probability( mPopulationStats.size() );

            for ( unsigned long i = 0; i < mPopulationStats.size(); i++ ) {
                probability[i] = mPopulationStats[i]*100/mEntries;
            }

            return probability;
        }
    };
}

#endif // __DD_BIT_SELECT_STRATEGY__
