/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_CALC_MODULE__
#define __DD_CALC_MODULE__

#include <cstring>
#include <gmp.h>
#include <gmpxx.h>

namespace double_description {

	class CalcModule {

		private:
		mpz_class register_x;
		mpz_class register_y;

		mpz_class register_s;
		mpz_class register_s1;
		mpz_class register_s2;

		mpz_class register_t;
		mpz_class register_t1;
		mpz_class register_t2;

		mpz_class register_g;
		mpz_class register_r;

		mpz_class register_p;
		mpz_class register_q;

		public:

		void calcLCM ( const mpz_class& a, const mpz_class& b ) {

			if ( b != 0 && a != 0 ) {
				register_g = a * b;
				calcGCD( a, b );
				register_y = register_g / register_y;
			} else if ( b != 0 ) {
				register_y = b;
			} else {
				register_y = 0;
			}
		}

		void startIncrementalLCM ( const mpz_class& initial ) {
			register_y = initial;
		}

		void calcIncrementalLCM ( const mpz_class& next ) {

			if ( next != 0 && register_y != 0 ) {
				register_g = register_y * next;
				calcIncrementalGCD( next );
				register_y = register_g/register_y;
			} else if ( next != 0 ) {
				register_y = next;
			}
		}
		
		void calcGCD ( const mpz_class& a, const mpz_class& b ) {

			if ( b == 0 ) {
				register_y = a;
			} else if ( a == 0 ) {
				register_y = b;
			} else {

				if ( abs( a ) >= abs( b ) ) {
					register_x = abs( a );
					register_y = abs( b );
				} else {
					register_x = abs( b );
					register_y = abs( a );
				}

				while ( register_x % register_y != 0 ) {
		//			register_g = register_x/register_y;
					register_r = register_x % register_y;

					register_x = register_y;
					register_y = register_r; 
				}		
			}
		}

		void calcExendedGCD ( const mpz_class& a, const mpz_class& b );

		void startIncrementalGCD ( const mpz_class& initial ) {
			register_y = initial;
		}
		
		void calcIncrementalGCD ( const mpz_class& next ) {

			register_x = next;

			while (  register_x != 0 ) {
				register_s = register_y % register_x;
				register_y = register_x;
				register_x = register_s;
			} 
		}

		const mpz_class& getResult () const {
			return register_y;
		}
		
		const mpz_class& getRegisterS () const {
			return register_s;
		}
		
		const mpz_class& getRegisterT () const {
			return register_t;
		}

		const mpz_class& getRegisterP () const {
			return register_p;
		}
		
		const mpz_class& getRegisterQ () const {
			return register_q;
		}

		void loadRegisterP ( const mpz_class& value ) {
			register_p = value;
		}

		void loadRegisterP ( const long value ) {
			register_p = value;
		}

		void divideRegisterP ( const mpz_class& divisor ) {
			register_p /= divisor;
		}

		void loadRegisterQ ( const mpz_class& value ) {
			register_q = value;
		}

		void loadRegisterQ ( const long value ) {
			register_q = value;
		}

		void divideRegisterQ ( const mpz_class& divisor ) {
			register_q /= divisor;
		}

		void loadRegisterS ( const mpz_class& value ) {
			register_s = value;
		}

		void loadRegisterS ( const long value ) {
			register_s = value;
		}

		void multiplyRegisterS ( const mpz_class& scalar ) {
			register_s *= scalar;
		}

		void subtractRegisterS ( const mpz_class& value ) {
			register_s -= value;
		}

		void divideRegisterS ( const mpz_class& divisor ) {
			register_s /= divisor;
		}

		void negateRegisterS () {
			register_s = -register_s;
		}
		
		void incrementRegisterS ( const mpz_class& increment ) {
			register_s += increment;
		}
	
		void loadRegisterT ( const mpz_class& value ) {
			register_t = value;
		}

		void loadRegisterT ( const long value ) {
			register_t = value;
		}
		
		void incrementRegisterT ( const mpz_class& increment ) {
			register_t += increment;
		}

		void subtractRegisterT ( const mpz_class& value ) {
			register_t -= value;
		}

		void multiplyRegisterT ( const mpz_class& scalar ) {
			register_t *= scalar;
		}

		void convertResultToAbsoluteValue () {
			register_y = abs( register_y );
		}

		void convertResultToNegativeValue () {
			register_y = -abs( register_y );
		}
	};
}
#endif // __DD_GDC_MODULE__
