/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_DIFFERENTIAL_BIT_STRATEGY__
#define __DD_DIFFERENTIAL_BIT_STRATEGY__

#include <vector>

#include "DDBitSelectStrategy.h"

namespace double_description {

    class DifferentialBitStrategy : public BitSelectStrategy {

        protected:
        const unsigned long mNegEntries;
        const std::vector<unsigned long> mNegPopulationStats;
        
        virtual double getProbabilityCoeff ( const unsigned long index,
                                             const double pos_probability,
                                             const double neg_probability ) const = 0;

        public:
        DifferentialBitStrategy ( const RayReferenceContainer & pos_entries,
                                  const RayReferenceContainer & neg_entries,
                                  const unsigned int plane ) :
            BitSelectStrategy( pos_entries, plane ),
            mNegEntries( neg_entries.size() ),
            mNegPopulationStats( createPopulationStats( neg_entries )) {}     

        std::vector<unsigned int> createMapping () const {

            std::vector<unsigned int> mapping;
            mapping.reserve( mPlane );

            const std::vector<unsigned long> & pos_population_stats = getPopulationStats();

            for ( unsigned int index = 0; index < mPlane; index++ ) {
 
                unsigned int position = 0;

                for ( unsigned int prev = 0; prev < index; prev++ ) {
                    if ( getProbabilityCoeff( prev, (double)pos_population_stats[prev]/mEntries, (double)mNegPopulationStats[prev]/mNegEntries ) >  getProbabilityCoeff( index, (double)pos_population_stats[index]/mEntries, (double)mNegPopulationStats[index]/mNegEntries ) ) {
                        position++;
                    }
                }

                mapping.insert( mapping.begin() + position, index );
            }

            return mapping;
        }
    };
}

#endif // __DD_DIFFERENTIAL_BIT_STRATEGY__
