/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_INPUT__
#define __DD_INPUT__

#include <fstream>

#include "DDMatrix.h"
#include "DDRankCalculator.h"

namespace double_description {
	
	class Input {

		private:
		const Matrix mSystem;
		const Matrix transformator;
		const Matrix transformed; 

        const Matrix getMatrixFromFile ( std::ifstream & input ) const;
		const Matrix getMatrixFromFile ( const char* file ) const;
        
		const Matrix applyNemhauserWolsey ( Matrix& matrix ) const;
		const Matrix applyPositiveBaseTransformation ( const Matrix& matrix ) const;

		static Matrix getIdentityMatrix ( const unsigned long dimension ) {
			Matrix identity( dimension, dimension );

			for ( unsigned long i = 0; i < dimension; i++ ) {
				identity.setValue( i, i, 1 );
			}

			return identity;
		}

		void findBasis ( Matrix& system ) const;

		public:
		Input ( const char* file );

		const Matrix & getSystem () const {
			return mSystem;
		}

		const Matrix & getTransformedSystem () const {
			return transformed;
		} 

		const Matrix & getTransformator () const {
			return transformator;
		} 
	};
}

#endif // __DD_INPUT__
