/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_INTERVAL__
#define __DD_INTERVAL__

#include <string>
#include <sstream>

namespace double_description {

    class Interval {
        
        private:
        unsigned int mLowerBound;
        unsigned int mUpperBound;

        public:
        Interval () :
            mLowerBound( 1 ),
            mUpperBound( 0 ) {}

        Interval ( const unsigned short lb,
                        const unsigned short ub ) :
            mLowerBound( lb ),
            mUpperBound( ub ) {}

        Interval ( const Interval & other ) :
            mLowerBound( other.mLowerBound ),
            mUpperBound( other.mUpperBound ) {}

        unsigned int & lb () {
            return mLowerBound;
        }

        const unsigned int & lb () const {
            return mLowerBound;
        }

        unsigned int & ub () {
            return mUpperBound;
        }

        const unsigned int & ub () const {
            return mUpperBound;
        }

        bool empty () const {
            return lb() > ub();
        }

        bool pointed () const {
            return lb() == ub();
        }

        void update ( const unsigned short value ) {

            if ( unlikely( empty() ) ) {
                lb() = value;
                ub() = value;
            } else if ( value > ub() ) {
                ub() = value;
            } else if ( value < lb() ) {
                lb() = value;
            }
        }

        void reset () {
            mLowerBound = 1;
            mUpperBound = 0;
        }

        const std::string toString () const {
            std::stringstream result;

            if ( empty() ) {
                result << "[empty]";
            } else {
                result << "[" << lb() << ".." << ub() << "]";
            }

            return result.str();
        }
    };
}

#endif // __DD_INTERVAL__
