/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_SINGLE_SET_RAY_GENERATOR__
#define __DD_SINGLE_SET_RAY_GENERATOR__

#include "DDMatrix.h"
#include "DDRayCache.h"

#include "DDExtremeRayUncompressor.h"

#ifdef __BIT_PATTERN_TREES__
    #include "DDBitPatternTreeCacheEntry.h"
#else 
    #ifdef __POPULATION_TREES__
        #include "DDPopulationTreeCacheEntry.h"
    #else
        #include "DDRayCacheEntry.h"    
        #include "DDPatternGraph.h"
    #endif
#endif

#include "DDRankCalculator.h"

#ifndef __CANDIDATES_BUFFER_SIZE__
#define __CANDIDATES_BUFFER_SIZE__ 67108864
#endif // __CANDIDATES_BUFFER_SIZE__

namespace double_description {

	class SingleSetRayGenerator {

		private:
#ifdef __COMPRESSION__
		const Matrix& mOriginal;
#endif
		const Matrix& mSystem;

#ifdef __BIT_PATTERN_TREES__
		RayCache<SingleSetRay, BitPatternTreeCacheEntry<SingleSetRay> > mCache;
#else
	#ifdef __POPULATION_TREES__
		RayCache<SingleSetRay, PopulationTreeCacheEntry<SingleSetRay> > mCache;
	#else
		RayCache<SingleSetRay, RayCacheEntry<SingleSetRay> > mCache;
	#endif // __POPULATION_TREES__
#endif // __BIT_PATTERN_TREES__

		RankCalculator mRankCalculator;

	//	PatternGraph mGraph; @ToDo Check if need this thing at all.

		std::vector<const SingleSetRay*> mCandidatesBuffer; 

#ifdef __COMPRESSION__
		std::vector<RepresentationVector> decompressed_rays;
#endif

		void calculateFirstStep ();
		void calculateStep ( const unsigned long plane );

		public:
#ifdef __MULTITHREADING__
		SingleSetRayGenerator ( const Matrix& system, const Matrix& transformator, const unsigned short threads );
#else
#ifdef __COMPRESSION__
		SingleSetRayGenerator ( const Matrix& original, const Matrix& system, const Matrix& transformator );
#else
		SingleSetRayGenerator ( const Matrix& system, const Matrix& transformator );
#endif
#endif
        
        void calculateBase ();
        void run ();

        template <class VECTOR>
        const std::vector<VECTOR> getExtremeRays () {
	
#ifdef __COMPRESSION__

	        ExtremeRayUncompressor decompressor( mSystem.getRows(), mSystem.getColumns(), BitList::getBlocks(mSystem.getRows()));
	        std::vector<const SingleSetRay*> & compressed_rays = mCache.getCompressedResults();
            std::vector<VECTOR> decompressed_rays;

	        while ( !compressed_rays.empty() ) {
                VECTOR current( mSystem.getColumns() );
                const  SingleSetRay * const current_compressed = compressed_rays.back();
		        decompressor.calcExtremeRay( mOriginal, current_compressed, current );

                decompressed_rays.push_back( current );    
                compressed_rays.pop_back();
	        }

	        return decompressed_rays;
#else
	        return mCache.getResults();
#endif
        }
	};
}

#endif // __DD_SINGLE_SET_RAY_GENERATOR_
