/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_VERIFICATION_CONDITION__
#define __DD_VERIFICATION_CONDITION__

#include "DDBitList.h"
#include "DDMapCondition.h"

namespace double_description {

	class VerificationCondition : virtual public MapCondition {

        private:
        const unsigned short mDegeneracy;

		public:
#ifdef __STATS_OUTPUT__
        static unsigned long OPS;
        static unsigned long CANDIDATES;

        static void start_new_stat_session () {
            OPS = 0;
            CANDIDATES = 0;
        }
#endif

		VerificationCondition ( const BitList & faceActiveSet ) :
			MapCondition( faceActiveSet ),
            mDegeneracy( faceActiveSet.getPopulationCount() ) {

#ifdef __STATS_OUTPUT__
            CANDIDATES++;
#endif    
        }

		virtual ~VerificationCondition() {}

		bool holdsOnFace ( const BitList & activeSetUnion ) const {
#ifdef __STATS_OUTPUT__
            VerificationCondition::OPS++;
#endif
			return activeSetUnion.contains( mMap );
		}

		virtual bool holdsOnFace ( const BitList & activeSetUnion, const BitList &, const BitList & ) const {
			return holdsOnFace( activeSetUnion );
		}

        bool holdsOnFace ( const BitList & activeSetUnion, const unsigned short degeneracy ) const {
            return mDegeneracy < degeneracy && holdsOnFace( activeSetUnion );
        }

        bool holdsOnFace( const BitList & activeSetUnion, const BitList & pattern_1, const BitList & pattern_2, const unsigned short degeneracy ) const {
            return mDegeneracy < degeneracy && holdsOnFace( activeSetUnion, pattern_1, pattern_2 );
        }

		bool holdsOnRay ( const BitList & activeSet ) const {
#ifdef __STATS_OUTPUT__
            VerificationCondition::OPS++;
#endif
			return activeSet.contains( mMap );
		}

        unsigned short pop_cnt() const {
            return mDegeneracy;
        }
	};
}

#endif // __DD_VERIFICATION_CONDITION__
