/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <iostream>
#include <stdlib.h>
#include <sys/time.h>

#include "DDInput.h"
#include "DDOutput.h"
#include "DDRepresentationVector.h"
#include "DDSingleSetRayGenerator.h"

#include "DDLogging.h"

#include "DDOptions.h"

using namespace double_description;

template <class GENERATOR>
int generate ( int argc, const char** argv ) {

    DD_OUTPUT("***********************************************************************************" )
    DD_OUTPUT("* Addibit: Another Double Description Implementation with BInary Trees.           *" )
    DD_OUTPUT("* Copyright (C) 2013  Blagoy Genov                                                *" )
    DD_OUTPUT("* This program comes with ABSOLUTELY NO WARRANTY!!!                               *" )
    DD_OUTPUT("***********************************************************************************\n" )

	if ( argc < 2 || strncmp( argv[1], "--help", 6 ) == 0 || strncmp( argv[1], "-help", 5 ) == 0 || strncmp( argv[1], "-h", 2 ) == 0 ) {
		DD_OUTPUT( "Usage: addibit MATRIX_FILE [CONFIGURATION_FILE]" );
		return -1;
	}

	struct timeval start, end;
	long mtime, seconds, useconds;

    Options & options = Options::current();

    if ( argc > 2 ) {
        options.readSettingsFile( argv[2] );
    }

    DD_OUTPUT( options.toString() )

	Input input ( argv[1] );

    DD_OUTPUT ( "-----------------------------------------------------------------------------------" )
    DD_OUTPUT ( "|                                  COMPUTATION                                    |" )
    DD_OUTPUT ( "-----------------------------------------------------------------------------------" )

#ifdef __MULTITHREADING__
	GENERATOR generator( input.getTransformedSystem(), input.getTransformator(), atoi( argv[2] ) );
#else
#ifdef __COMPRESSION__
	GENERATOR generator( input.getSystem(), input.getTransformedSystem(), input.getTransformator() );
#else
	GENERATOR generator( input.getTransformedSystem(), input.getTransformator() );
#endif
#endif

	gettimeofday(&start, NULL);	


#ifdef __STATS_OUTPUT__
    if ( options.plot_required() ) {
        OPEN_LOG( COMPLEXITY_PLOT )
        OPEN_LOG( VER_BIT_DISTRIBUTION_PLOT )
    }
#endif

    generator.calculateBase();

    START_TIMER( ITERATIONS_TIME )
    generator.run();
    STOP_TIMER( ITERATIONS_TIME )

#ifdef __STATS_OUTPUT__
    if ( options.plot_required() ) {
        DUMP_LOG( COMPLEXITY_PLOT, options.plot_file().c_str() )
        DUMP_LOG( VER_BIT_DISTRIBUTION_PLOT, "ver_bit_distribution.plt" )
    }
#endif

    if ( options.output_format() == INTEGER_VEC ) {
    	Output::write<RepresentationVector>( argv[1], generator.template getExtremeRays<RepresentationVector>() );
    } else if ( options.output_format() == DOUBLE_VEC ) {
        Output::write<OutputVector<double> >( argv[1], generator.template getExtremeRays<OutputVector<double> >() );
    } else {
        Output::write<BitList>( argv[1], generator.template getExtremeRays<BitList>() );
    }

	gettimeofday(&end, NULL);

	seconds  = end.tv_sec  - start.tv_sec;
	useconds = end.tv_usec - start.tv_usec;

	mtime = ((seconds) * 1000 + useconds/1000.0) + 0.5;

    DD_OUTPUT( "Narrowing and verification time: " << TIMER_VALUE( ITERATIONS_TIME ) << "ms" )
	DD_OUTPUT( "Overall time: " << mtime << "ms" );

    Options::release();

	return 0;
}
